import re
import json
import requests
from typing import Any

def _standardize_url(url):
    if url.startswith("http://") or url.startswith("https://"):
        return url
    else:
        return "http://" + url
    
def regex(pattern, response=None):
    match = re.search(pattern, response)
    if match:
        return {"result": True, "reason": f"regex pattern {pattern} found in output"}
    else:
        return {
            "result": False,
            "reason": f"regex pattern {pattern} not found in output",
        }

def contains_any(keywords, response=None, case_sensitive=False):
    if isinstance(keywords, str):
        keywords = keywords.split(",")
    keywords = list(map(lambda k: k.strip(), keywords))
    if not case_sensitive:
        response = response.lower()
        keywords = list(map(lambda k: k.lower(), keywords))

    found_keywords = []
    for keyword in keywords:
        if keyword in response:
            found_keywords.append(keyword)

    if found_keywords:
        result = True
        reason = f"One or more keywords were found in output: " + ", ".join(
            found_keywords
        )
    else:
        result = False
        reason = "No keywords found in output"

    return {"result": result, "reason": reason}

def contains_json(response=None):
    try:
        json.loads(response)
        result = True
    except json.JSONDecodeError:
        result = False
    if result:
        return {
            "result": True,
            "reason": "Output contains JSON",
        }
    else:
        return {
            "result": False,
            "reason": "Output does not contain JSON",
        }

def no_invalid_links(response=None):
    pattern = r"(?!.*@)(?:https?://)?(?:www\.)?\S+\.\S+"
    link_match = re.search(pattern=pattern, string=response)
    if link_match:
        matched_url = link_match.group()
        if matched_url:
            standardized_url = _standardize_url(matched_url)
            try:
                response = requests.head(standardized_url)
                if response.status_code == 200:
                    return {
                        "result": True,
                        "reason": f"link {matched_url} found in output and is valid",
                    }
                else:
                    return {
                        "result": False,
                        "reason": f"link {matched_url} found in output but is invalid",
                    }
            except:
                return {
                    "result": False,
                    "reason": f"link {matched_url} found in output but is invalid",
                }
    return {"result": True, "reason": f"no link found in output"}

def api_call(
    url: str,
    payload: dict,
    response = None,
    headers: dict = None,
):
    payload["response"] = response
    api_response = requests.post(url, json=payload, headers=headers)
    result = api_response.json().get("result")
    reason = api_response.json().get("reason")
    return {
        "result": result,
        "reason": reason,
    }

operations = {
    "Regex": regex,
    "ContainsAny": contains_any,
    "ContainsJson": contains_json,
    "NoInvalidLinks": no_invalid_links,
}




from enum import Enum


class LlmEvalTypeId(Enum):
    CONTEXT_CONTAINS_ENOUGH_INFORMATION = "Ccei"
    DOES_RESPONSE_ANSWER_QUERY = "Draq"
    FAITHFULNESS = "Irftc"
    GRADING_CRITERIA = "GradingCriteria"
    CUSTOM_PROMPT = "CustomPrompt"
    SUMMARY_ACCURACY = "SummaryAccuracy"

class RagasEvalTypeId(Enum):
    RAGAS_CONTEXT_RELEVANCY = "RagasContextRelevancy"
    RAGAS_ANSWER_RELEVANCY = "RagasAnswerRelevancy"

class FunctionEvalTypeId(Enum):
    REGEX = "Regex"
    CONTAINS_ANY = "ContainsAny"



from typing import Optional, List
from athina.evals.eval_type import FunctionEvalTypeId
from athina.evals.function.function_evaluator import FunctionEvaluator


class ContainsAny(FunctionEvaluator):
    def __init__(
        self,
        keywords: List[str],
        case_sensitive: Optional[bool] = False,
    ):
        super().__init__(
            function_name=FunctionEvalTypeId.CONTAINS_ANY.value,
            function_arguments={"keywords":keywords, "case_sensitive":case_sensitive},
        )

class Regex(FunctionEvaluator):
    def __init__(
        self,
        regex: str,
    ):
        super().__init__(
            function_name=FunctionEvalTypeId.REGEX.value,
            function_arguments={"pattern":regex},
        )



from typing import Optional, List
from athina.metrics.metric_type import MetricType
import time
from typing import Optional
from athina.interfaces.result import EvalResult, EvalResultMetric
from athina.helpers.logger import logger
from athina.interfaces.athina import AthinaExperiment
from ..base_evaluator import BaseEvaluator
from .functions import operations

class FunctionEvaluator(BaseEvaluator):

    _function_name: str
    _function_arguments: dict

    """
    This evaluator runs the requested Function on the given data.
    """

    @property
    def _model(self):
        return None
    
    @property
    def name(self):
        return self._function_name

    @property
    def display_name(self):
        return self._function_name

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.PASSED.value]

    @property
    def default_function_arguments(self):
        return {}

    @property
    def required_args(self):
        return ["response"]

    @property
    def examples(self):
        return None

    def __init__(
        self,
        function_name: Optional[str] = None,
        function_arguments: Optional[dict] = None,
    ):
        if function_name is None:
            raise ValueError(f"function_name is a required argument")
        if function_arguments is None:
            function_arguments = self.default_function_arguments
        if function_name not in operations.keys():
            raise ValueError(f"Unsupported function: {function_name}")
        else:
            self._function_name = function_name
            self._function_arguments = function_arguments


    def _evaluate(self, **kwargs) -> EvalResult:
        """
        Run the Function evaluator.
        """
        start_time = time.perf_counter()

        # Validate that correct args were passed
        self.validate_args(**kwargs)
        metrics = []
        try: 
            # Evaluate the dataset using Function
            operator = operations.get(self._function_name)
            response = operator(**kwargs, **self._function_arguments)
            metrics.append(EvalResultMetric(id=MetricType.PASSED.value, value=float(response["result"])))
            explanation = response['reason']

        except Exception as e:
            logger.error(f"Error occurred during eval: {e}")
            raise e

        end_time = time.perf_counter()
        eval_runtime_ms = int((end_time - start_time) * 1000)
        eval_result = EvalResult(
            name=self.name,
            display_name=self.display_name,
            data=kwargs,
            reason=explanation,
            runtime=eval_runtime_ms,
            model=None,
            metrics=metrics,
            failure=not response["result"] if response is not None and "result" in response else None,
        )
        return {k: v for k, v in eval_result.items() if v is not None}




from enum import Enum
from .agreement_score import AgreementScore
from .hallucination_score import HallucinationScore
from .contradiction_score import ContradictionScore
from .ragas_context_relevancy import RagasContextRelevancy
from .ragas_answer_relevancy import RagasAnswerRelevancy
from .passed import Passed
from .metric import Metric

class MetricType(Enum):
    AGREEMENT_SCORE = "agreement_score"
    HALLUCINATION_SCORE = "hallucination_score"
    CONTRADICTION_SCORE = "contradiction_score"
    RAGAS_CONTEXT_RELEVANCY = "ragas_context_relevancy"
    RAGAS_ANSWER_RELEVANCY = "ragas_answer_relevancy"
    PASSED = 'passed'

    @staticmethod
    def get_class(metric_type):
        """
        Returns the class of the metric type.
        """
        if metric_type == MetricType.AGREEMENT_SCORE.value:
            return AgreementScore
        elif metric_type == MetricType.HALLUCINATION_SCORE.value:
            return HallucinationScore
        elif metric_type == MetricType.CONTRADICTION_SCORE.value:
            return ContradictionScore
        elif metric_type == MetricType.RAGAS_CONTEXT_RELEVANCY.value:
            return RagasContextRelevancy
        elif metric_type == MetricType.RAGAS_ANSWER_RELEVANCY.value:
            return RagasAnswerRelevancy
        elif metric_type == MetricType.PASSED.value:
            return Passed
        else:
            raise NotImplementedError(f"Metric type {metric_type} not implemented.")



# athina/evals/__init__.py
from athina.evals.llm.does_response_answer_query.evaluator import (
    DoesResponseAnswerQuery,
)
from athina.evals.llm.context_contains_enough_information.evaluator import (
    ContextContainsEnoughInformation,
)
from athina.evals.base_evaluator import BaseEvaluator
from athina.evals.llm.faithfulness.evaluator import Faithfulness
from athina.evals.llm.grading_criteria.evaluator import GradingCriteria
from athina.evals.llm.custom_prompt.evaluator import CustomPrompt
from athina.evals.llm.summary_accuracy.evaluator import SummaryAccuracy
from athina.evals.ragas.context_relevancy.evaluator import RagasContextRelevancy
from athina.evals.ragas.answer_relevancy.evaluator import RagasAnswerRelevancy
from athina.evals.function.function_evaluator import FunctionEvaluator
from athina.evals.llm.llm_evaluator import LlmEvaluator
from athina.evals.function.wrapper import ContainsAny
from athina.evals.function.wrapper import Regex

__all__ = [
    "BaseEvaluator",
    "LlmEvaluator",
    "DoesResponseAnswerQuery",
    "SummaryAccuracy",
    "ContextContainsEnoughInformation",
    "Faithfulness",
    "RagasContextRelevancy",
    "RagasAnswerRelevancy"
    "FunctionEvaluator",
    "GradingCriteria",
    "CustomPrompt",
    "ContainsAny",
    "Regex"
]
